function [Hes,Jac,grad]=construct_jacobian_caf_C2(L,T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=size(L{1},1);
R=size(L{3},1);
K=size(L{1},2);
D=[N N R 4];

A=L{1};
B=L{2};
G1=L{3};
G2=L{4};
G3=L{5};

P=N*N*R;
Q=(2*N+3*R)*K;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%JAC1%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize Jacobian matrix
Jac1= zeros(P,Q);
for p=1:P
    k=floor((p-1)/(N*N))+1;
    x=p-(k-1)*N*N;
    j=floor((x-1)/N)+1;
    i=x-(j-1)*N;  
    if i==j
        for t=1:K
            Jac1(p,(i-1)*K+t)=2*A(i,t)*G1(k,t)-2*B(i,t)*G2(k,t);
            Jac1(p,N*K+(j-1)*K+t)=-2*A(i,t)*G2(k,t)+2*B(i,t)*G3(k,t);
        end
    else
        for t=1:K
            Jac1(p,(i-1)*K+t)=A(j,t)*G1(k,t)-B(j,t)*G2(k,t);
            Jac1(p,(j-1)*K+t)=A(i,t)*G1(k,t)-B(i,t)*G2(k,t);
            Jac1(p,N*K+(i-1)*K+t)=B(j,t)*G3(k,t)-A(j,t)*G2(k,t);
            Jac1(p,N*K+(j-1)*K+t)=B(i,t)*G3(k,t)-A(i,t)*G2(k,t);
        end
    end   
    for t=1:K
        Jac1(p,2*N*K+(k-1)*K+t)=A(i,t)*A(j,t);
        Jac1(p,2*N*K+R*K+(k-1)*K+t)=-A(i,t)*B(j,t)-B(i,t)*A(j,t);;
        Jac1(p,2*N*K+2*R*K+(k-1)*K+t)=B(i,t)*B(j,t);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%JAC2%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize Jacobian matrix
Jac2= zeros(P,Q);
for p=1:P
    k=floor((p-1)/(N*N))+1;
    x=p-(k-1)*N*N;
    j=floor((x-1)/N)+1;
    i=x-(j-1)*N;  
    if i==j
        for t=1:K
            Jac2(p,(i-1)*K+t)=2*A(i,t)*G3(k,t)+2*B(i,t)*G2(k,t);
            Jac2(p,N*K+(j-1)*K+t)=2*A(i,t)*G2(k,t)+2*B(i,t)*G1(k,t);
        end
    else
        for t=1:K
            Jac2(p,(i-1)*K+t)=A(j,t)*G3(k,t)+B(j,t)*G2(k,t);
            Jac2(p,(j-1)*K+t)=A(i,t)*G3(k,t)+B(i,t)*G2(k,t);
            Jac2(p,N*K+(i-1)*K+t)=B(j,t)*G1(k,t)+A(j,t)*G2(k,t);
            Jac2(p,N*K+(j-1)*K+t)=B(i,t)*G1(k,t)+A(i,t)*G2(k,t);
        end
    end   
    for t=1:K
        Jac2(p,2*N*K+(k-1)*K+t)=B(i,t)*B(j,t);
        Jac2(p,2*N*K+R*K+(k-1)*K+t)=B(i,t)*A(j,t)+A(i,t)*B(j,t);
        Jac2(p,2*N*K+2*R*K+(k-1)*K+t)=A(i,t)*A(j,t);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%JAC3%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize Jacobian matrix
Jac3= zeros(P,Q);
for p=1:P
    k=floor((p-1)/(N*N))+1;
    x=p-(k-1)*N*N;
    j=floor((x-1)/N)+1;
    i=x-(j-1)*N;  
    if i==j
        for t=1:K
            Jac3(p,(i-1)*K+t)=2*A(i,t)*G2(k,t)+B(i,t)*(G1(k,t)-G3(k,t));
            Jac3(p,N*K+(j-1)*K+t)=-2*B(i,t)*G2(k,t)+A(i,t)*(G1(k,t)-G3(k,t));
        end
    else
        for t=1:K
            Jac3(p,(i-1)*K+t)=B(j,t)*G1(k,t)+A(j,t)*G2(k,t);
            Jac3(p,(j-1)*K+t)=A(i,t)*G2(k,t)-B(i,t)*G3(k,t);
            Jac3(p,N*K+(i-1)*K+t)=-B(j,t)*G2(k,t)-A(j,t)*G3(k,t);
            Jac3(p,N*K+(j-1)*K+t)=A(i,t)*G1(k,t)-B(i,t)*G3(k,t);
        end
    end   
    for t=1:K
        Jac3(p,2*N*K+(k-1)*K+t)=A(i,t)*B(j,t);
        Jac3(p,2*N*K+R*K+(k-1)*K+t)=A(i,t)*A(j,t)-B(i,t)*B(j,t);;
        Jac3(p,2*N*K+2*R*K+(k-1)*K+t)=-B(i,t)*A(j,t);
    end
end
Jac4=[];
Jac=[Jac1;Jac2;Jac3;Jac4];

%P=4*P;
P=3*P;
Hes=Jac'*Jac;
Td=reshape(T,P,1);
[Te]=model_caf_C2(L);
Tde=reshape(Te,P,1);
err= Tde - Td;
grad=Jac'*err;



