function [L_newv,res,lda_new,nu,Hess,grad,Jac,flag,gain]=boucle_lm_opt4(L,T,lda_old,nu,Hess,grad,Jac,eps2,eps1,flag)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute one iteration of the LM method with els

L_newv=L;
F= size(L{1},2);
I=size(T); % tensor dimensions
D=length(I); % tensor order (number of dimensions)

% reshape data tensor
Td=reshape(T,prod(I),1);
Td2=reshape(T,[I(1) prod(I(2:end))]);
N=size(Td2,2);
K=cell(D,2);
% old error and cost function value
p_old= zeros(sum(I)*F,1);
x_old= ones(1,F);
k=1;
for i=1:D
    p_old(k:k-1+I(i)*F)=vec(L{i}.');
    x_old=pkr(L{i},x_old);
    k=k+I(i)*F;
end
e_old= x_old*ones(F,1) - Td;
f_old= e_old'*e_old/(2);

% step
dp= -inv(Hess + lda_old*I_(sum(I)*F))*grad;

k=1;
for i=1:D
    K{i,1}=unvec(dp(k:k-1+I(i)*F),F,I(i)).';
    k=k+I(i)*F;
    K{i,2}=L{i};
end
mu=els(Td2,K,N,F);
dp=mu*dp;

if norm(dp)<eps2*(norm(p_old)+eps2);
    flag=1;
    %f_new=f_old;
    e_new=e_old;
    lda_new=lda_old;
    gain=0;
else 
    % update p-vector
    p_new= p_old + dp;
    % rebuild factor matrices
    k=1;
    for i=1:D
        L_new{i}=unvec(p_new(k:k-1+I(i)*F),F,I(i)).';
        k=k+I(i)*F;
    end

    % new error and cost function value
    x_new=ones(1,F);
    for i=1:D
        x_new=pkr(L_new{i},x_new);
    end
    e_new= x_new*ones(F,1) - Td;
    f_new= e_new'*e_new/(2);
    Jp=(Jac*dp)';
    f_new_approx = f_old + Jp*e_old + Jp*Jp'/2;
    % calculate gain ratio
    gain= ((f_old - f_new)/(f_old - f_new_approx));
    
    
    % update lambda (regularization factor)
    if gain >0
        lda_new=lda_old*max(1/3,1-(2*gain-1)^3);
        nu=2;
        L_newv=L_new;
        [Hess,Jac]=construct_hessian(L_new,T);
        grad=construct_gradient(L_new,T);
        flag=(max(grad)<eps1);
    else       
        lda_new=lda_old*nu;
        nu=2*nu;
        %f_new=f_old;
        e_new=e_old;
    end

end
res=e_new'*e_new/sum((Td(:)).^2);
