function [Data,pData]=create_data(typeS,complexH,nbs,nbc,nbo,snr,num)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Creates simulated sources, mixing and observations matrices.
%typeS: Sources constellation type (ex '2psk','4qam'...)
%complexH: 1 for a complex mixing matrix or 0 for a real one.
%nbs: sources number
%nbc: mixtures number 
%nbo: observations number (inf simulates an infinite number of observation)
%snr: signal to noise ratio (if set to inf, no noise is added) 
%num: number of the save files
%pData stored the parameters values
%Observation matrix is stored in Data{1}
%Sources matrix is stored in Data{2}
%Mixing matrix is stored in Data{3}
%Data and parameters are saved in Data'num' pData'num' respectively (ex of save files names: Data1 an pData1 if
%num=1).

% Create sources matrix S
if strcmp(typeS,'2psk');
    if nbo==inf
        S=ones(nbs,2^nbs);
        for nl=1:nbs
            for nc=1:2^nl:2^(nbs)-2^nl+1;
                S(nl,[nc:nc+2^(nl-1)-1])=-ones(1,2^(nl-1));
            end
        end
    else
        S=sign(rand(nbs,nbo)-0.5);
    end
else
    if  strcmp(typeS,'2qam')
        S=real(create_symbols(typeS,nbs,nbo));
    else
        S=(create_symbols(typeS,nbs,nbo));
    end
end


%Create mixing matrix H with uncorelated, norm 1, columns 
if nbs<=nbc*(complexH+1);
    pvalmin=0.3;
else
    pvalmin=0.05;
end
pval=0;
while pval<pvalmin
    H=(randn(nbc,nbs)+j*complexH*randn(nbc,nbs));
    HH=zeros((complexH+1)*nbc,nbs);
    if complexH==1
        HH(1:2:end-1)=real(H);
        HH(2:2:end)=imag(H);
    else
        HH=H;
    end
    [r,p]=corrcoef(HH);
    pval=min(min(p));
end
for i=1:nbs
    H(:,i)=H(:,i)/norm(H(:,i));
end



%Create observation matrix 
if snr~=inf
    Xv=H*S;
    Px=Xv(:)'*Xv(:)/numel(Xv);
    B=((randn(size(Xv))+j*complexH*randn(size(Xv))))/sqrt(complexH+1);
    B=sqrt(Px*10^(-snr/10))*B;
    Pb=B(:)'*B(:)/numel(B);
    snrv=10*log10(Px/Pb);
    X=Xv+B;
    errXb=sum(sum(([real(X) imag(X)]-[real(Xv) imag(Xv)]).^2))/sum(sum([real(Xv) imag(Xv)].^2));
else
    X=H*S;
    snrv=inf;
    errXb=0;
end


Data{1}=X;
Data{2}=S;
Data{3}=H;


pData={typeS,complexH,nbs,nbc,nbo,snrv,snr,errXb};

save(['Data' num2str(num)],'Data')
save(['pData' num2str(num)],'pData')

