function [T]=create_gfT(Z,complexGF,nu,fac,ordre);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Create the caf derivation tensor T.
%Z is the observation matrix.
%complexGF: 1 if the mixing matrix is complex or 0 if it is real.
%ordre is the differentiation order. Ord can be 2, 3 or 4 in the complex case or
%3, 4 or 43 in the real case (43 merge order 3 and 4 in a 4 ways tensor).
%nu is the grid size.
%fac: grid points belong to [-fac; fac]^nbc.

nbc=size(Z,1);
a=-fac;
b=fac;
k=0;

switch complexGF
    case 1
        W=zeros(nbc,nu);
        ind=[nu 2*nbc*ones(1,ordre)];
        T=zeros(ind);
        for j=1:nu
            flag=1;
            while flag==1
                w=(b-a)*rand(nbc,1)+a+((b-a)*rand(nbc,1)+a)*i;
                expo=exp(real(w'*Z));
                phi=mean(expo);
                if (phi)>0
                    flag=0;
                    W(:,j)=w;
                    k=k+1;
                end
            end
            switch ordre
                case 4
                    T(j,:,:,:,:)=create_gfT_4(w,Z);
                    %TT{j}=T(j,:,:,:,:);
                case 3
                    T(j,:,:,:)=create_gfT_3(w,Z);
                    %TT{j}=T(j,:,:,:);
                case 2
                    T(j,:,:)=create_gfT_2(w,Z);
                    %TT{j}=T(j,:,:);
            end
        end
        rank(W);
 
    case 0
        U=zeros(nbc,nu);
        ind=[nu nbc*ones(1,ordre)];
        T=zeros(ind);
        if ordre==43;
            T=zeros(nu*(nbc+1),nbc,nbc,nbc);
        end
        for j=1:nu
            flag=1;
            while flag==1
                u=(b-a)*rand(nbc,1)+a;
                expo=exp(u.'*Z);
                phi=mean(expo);
                if phi>0
                    flag=0;
                    U(:,j)=u;
                    k=k+1;
                end
            end
            switch ordre
                case 4
                    T(j,:,:,:,:)=create_gfT_4r(u,Z);
                    %TT{j}=T(j,:,:,:,:);
                case 3
                    T(j,:,:,:)=create_gfT_3r(u,Z);
                    %TT{j}=T(j,:,:,:);
                case 43
                    T((j-1)*nbc+1:j*nbc,:,:,:)=create_gfT_4r(u,Z);
                    T((nbc*nu)+j,:,:,:)=create_gfT_3r(u,Z);
            end
        end
        rank(U);
end
       
