function [He,err,eH]=run_caf(X,nbs,nu,fac,ordre,nbtest,ini,tol,algo,niter,elsper,col,H);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%estimate the mixing matrix (He) and plot fit vs iteration number.

%err is the reconstruction error and eH is the NMSE between He and the real mixing matrix H (if H is given)

%X is the observation matrix.

%nbs is the number of sources

%nu: grid size.

%fac: grid points are chosen within [-fac; fac]^nbc.

%ordre is the differentiation order of the caf. ordre can be 2, 3 or 4 in the complex case or
%3, 4 or 43 in the real case (43 merge order 3 and 4 in a 4 ways tensor).

%nbtest: number of random initialisations to be compared (in the case of random initialisation). The best
%initialisation up to the initial fit is chosen.

%ini: Initialisation parameter. if ini=0, initial matrices are loaded from 
%file Lini.mat, othewise, random initialisation is performed and initial
%matrices are saved in Lini.mat.

%tol: Algorithm stop if the relative fit improvement between two successive iterations is less than
%tol.

%algo: string specifying the algorithm: 'als' for ALESCAF, 'grad' for
% GRADCAF, 'lm' for LEMACAF or 'nlcg' for CGRADCAF

%niter is the maximal number of iteration allowed to the algorithm

%elsper is the Enhanced Line Search cycle, 0 means no ELS

%col: string defining color, line type and plot symbols of the output plot.

%if given, H is the true mixing matrix to be compared to He in order
%to test the ALESCAF approach. Othewise one MUST NOT declare the H
%argument.


%Data parameters
%[nbc,nbo]=size(X);
if nargin==nargin('run_CAF')
    complexH=~isreal(H);
else 
    H=[];
    complexH=~isreal(X);
end


T=create_gfT(X,complexH,nu,fac,ordre);% Create the tensor (T) of the caf derivatives. Tensor order is ordre+1
[Le,err_old]=choix_ini(T,nbs,nbtest,ini);% Compute initialisation matrices of the PARAFAC decomposition. 
%These are stored in the cell array Le.
[He,eH,err]=parafac_dec(T,nbs,tol,complexH,algo,niter,elsper,Le,err_old,col,H);
%perform the PARAFAC decomposition of T.




